#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "global.h"

int verbose=0, verboseMemory=0, quiet=0, userSeed;
long totalMem=0;
unsigned long randomSeed;

/* functions */

/*************************************/
void *AllocMem(long n, char *name, char *func, int showInfo)
{
	void *P;
	
	if ( (P=malloc(n))==NULL ) {
		fprintf(stderr, "Out of memory allocating '%s': %s()\n", name, func);
		exit(0);
	}
	
	totalMem+=n;

	if (showInfo && verboseMemory)
		fprintf(stderr, "%s in %s() - %ld bytes\n", name, func, n);
	
	return P;
}


/*************************************/
void *CAllocMem(long n, char *name, char *func, int showInfo)
{
	void *P;
	
	if ( (P=calloc(n, 1))==NULL ) {
		fprintf(stderr, "Out of memory allocating '%s': %s()\n", name, func);
		exit(0);
	}
	
	totalMem+=n;
	if (showInfo && verboseMemory)
		fprintf(stderr, "%s in %s() - %ld bytes\n", name, func, n);
	
	return P;
}


/*************************************/
int GetDoubleParams(int argc, char **argv, int *argn, char *pos, int numParams, double *params)
{
	int i;
        /*CS:  I upped the buffer for the #ffreq_file# option for indel-Seq-Gen.  Since I normalize the frequencies
         * inside of iSG and output them using perl, the precision of the 20 amino acid frequencies is ~16 places,
         * making a total of 360 points of precision, after 0. is added to the front.  So, adding the commas, 380
         * total positions are needed, and another 20 positions just to be safe. */
        char *st, buf[400]; 
	
	i=0;
	strcpy(buf, pos);
	st=strtok(buf, "\t,/");
	do {
		if (st==NULL) {
			if ((*argn)+1<argc) {
				(*argn)++;
				strcpy(buf, argv[*argn]);
			} else
				return -1;
			st=strtok(buf, "\t,/");
			if (st==NULL)
				return -1;
		}
		
		if (sscanf(st, "%lf", params+i)!=1)
			return -1;
		i++;
		if (i<numParams)
			st=strtok(NULL, "\t,/");
	} while (i<numParams);

	return 0;
}


/*************************************/
int GetIntParams(int argc, char **argv, int *argn, char *pos, int numParams, int *params)
{
	int i;
	char *st, buf[256];
	
	i=0;
	strcpy(buf, pos);
	st=strtok(buf, "\t,/");
	do {
		if (st==NULL) {
			if ((*argn)+1<argc) {
				(*argn)++;
				strcpy(buf, argv[*argn]);
			} else
				return -1;
			st=strtok(buf, "\t,/");
			if (st==NULL)
				return -1;
		}
		
		if (sscanf(st, "%d", params+i)!=1)
			return -1;
		i++;
		if (i<numParams)
			st=strtok(NULL, "\t,/");
	} while (i<numParams);

	return 0;
}


/*************************************/
int GetUnsignedLongParams(int argc, char **argv, int *argn, char *pos, int numParams, unsigned long *params)
{
	int i;
	char *st, buf[256];
	
	i=0;
	strcpy(buf, pos);
	st=strtok(buf, "\t,/");
	do {
		if (st==NULL) {
			if ((*argn)+1<argc) {
				(*argn)++;
				strcpy(buf, argv[*argn]);
			} else
				return -1;
			st=strtok(buf, "\t,/");
			if (st==NULL)
				return -1;
		}
		
		if (sscanf(st, "%lu", params+i)!=1)
			return -1;
		i++;
		if (i<numParams)
			st=strtok(NULL, "\t,/");
	} while (i<numParams);

	return 0;
}


/*************************************/
int GetStrParam(int argc, char **argv, int *argn, char *pos, char *param, int len)
{
	int i;
	char *st, *P, buf[256];
	
	i=0;
	strcpy(buf, pos);
	st=strtok(buf, "\t,/");
	if (st==NULL) {
		if ((*argn)+1<argc) {
			(*argn)++;
			strcpy(buf, argv[*argn]);
		} else
			return -1;
		st=strtok(buf, "\t,/");
		if (st==NULL)
			return -1;
	}
	strncpy(param, st, len);
	param[len]='\0';
	P=param;
	while (*P) {
		*P=toupper(*P);
		P++;
	}
	
	return 0;
}
